@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012 Free Software Foundation, Inc.
+  Copyright (C) 2001-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -23,7 +22,6 @@
 # define IFCONFIG_IFCONFIG_H
 
 # include "flags.h"
-# include "if_index.h"
 # include "options.h"
 # include "printif.h"
 # include "system.h"
