@@ -1,6 +1,5 @@
 /* options.c -- process the command line options
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012 Free Software Foundation, Inc.
+  Copyright (C) 2001-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -37,11 +36,22 @@
 
 #include <sys/socket.h>
 #include <net/if.h>
+
+#include <attribute.h>
+
 #include "ifconfig.h"
 
+/* List available interfaces.  */
+int list_mode;
+
 /* Be verbose about actions.  */
 int verbose;
 
+/* Flags asked for, possibly still pending application.  */
+static int pending_setflags = 0;
+static int pending_clrflags = 0;
+static int pending_valid = 0;
+
 /* Array of all interfaces on the command line.  */
 struct ifconfig *ifs;
 
@@ -51,6 +61,7 @@ int nifs;
 static struct ifconfig ifconfig_initializer = {
   NULL,				/* name */
   0,				/* valid */
+  NULL, NULL, 0, NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL
 };
 
 struct format formats[] = {
@@ -62,7 +73,8 @@ struct format formats[] = {
   /* This is the standard GNU output.  */
   {"gnu",
    "Standard GNU output format.",
-   "${first?}{}{${\\n}}${format}{gnu-one-entry}"},
+   "${ifdisplay?}{${first?}{}{${\\n}}${format}{gnu-one-entry}}"
+  },
   {"gnu-one-entry",
    "Same as GNU, but without additional newlines between the entries.",
    "${format}{check-existence}"
@@ -72,11 +84,17 @@ struct format formats[] = {
    "${netmask?}{  netmask ${tab}{16}${netmask}${\\n}}"
    "${brdaddr?}{  broadcast ${tab}{16}${brdaddr}${\\n}}"
    "${dstaddr?}{  peer address ${tab}{16}${dstaddr}${\\n}}"
+   "${exists?}{tunnel?}{"
+     "${tunnel?}{  tunnel src ${tab}{16}${tunsrc}${\\n}}}"
+   "${exists?}{tunnel?}{"
+     "${tunnel?}{  tunnel dst ${tab}{16}${tundst}${\\n}}}"
    "${flags?}{  flags ${tab}{16}${flags}${\\n}}"
    "${mtu?}{  mtu ${tab}{16}${mtu}${\\n}}"
    "${metric?}{  metric ${tab}{16}${metric}${\\n}}"
    "${exists?}{hwtype?}{${hwtype?}{  link encap ${tab}{16}${hwtype}${\\n}}}"
    "${exists?}{hwaddr?}{${hwaddr?}{  hardware addr ${tab}{16}${hwaddr}${\\n}}}"
+   "${media?}{  link medium ${tab}{16}${media}${\\n}}"
+   "${status?}{  link status ${tab}{16}${status}${\\n}}"
    "${exists?}{txqlen?}{${txqlen?}{  tx queue len ${tab}{16}${txqlen}${\\n}}}"
    "}"
   },
@@ -88,12 +106,16 @@ struct format formats[] = {
    "${name}${exists?}{hwtype?}{${hwtype?}{${tab}{10}Link encap:${hwtype}}"
    "${hwaddr?}{  HWaddr ${hwaddr}}}${\\n}"
    "${addr?}{${tab}{10}inet addr:${addr}"
+   "${dstaddr?}{  P-t-P:${dstaddr}}"
    "${brdaddr?}{  Bcast:${brdaddr}}"
    "${netmask?}{  Mask:${netmask}}"
    "${newline}}"
    "${tab}{10}${flags}"
    "${mtu?}{  MTU:${mtu}}"
    "${metric?}{  Metric:${metric}}"
+   "${media?}{"
+   "${newline}${tab}{10}Media: ${media}"
+   "${status?}{, ${status}}}"
    "${exists?}{ifstat?}{"
    "${ifstat?}{"
    "${newline}          RX packets:${rxpackets}"
@@ -108,7 +130,6 @@ struct format formats[] = {
    "${newline}"
    "          RX bytes:${rxbytes}  TX bytes:${txbytes}"
    "}}{"
-   "${newline}"
    "${exists?}{txqlen?}{${txqlen?}{ ${tab}{10}txqueuelen:${txqlen}}${\\n}}}"
    "${newline}"
    "${exists?}{map?}{${map?}{${irq?}{"
@@ -126,27 +147,37 @@ struct format formats[] = {
    "${first?}{Iface    MTU Met    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg${newline}}"
    "${format}{check-existence}"
    "${name}${tab}{6}${mtu}{%6d} ${metric}{%3d}"
+   /* Insert blanks without ifstat.  */
+   "${exists?}{ifstat?}{"
    "${ifstat?}{"
    " ${rxpackets}{%8lu} ${rxerrors}{%6lu} ${rxdropped}{%6lu} ${rxfifoerr}{%6lu}"
    " ${txpackets}{%8lu} ${txerrors}{%6lu} ${txdropped}{%6lu} ${txfifoerr}{%6lu}"
-   "}{   - no statistics available -}"
+   "}{   - no statistics available -}}"
    "${tab}{76} ${flags?}{${flags}{short}}{[NO FLAGS]}"
    "${newline}"
   },
   /* Resembles the output of ifconfig shipped with unix systems like
      Solaris 2.7 or HPUX 10.20.  */
   {"unix",
-   "Traditional UNIX interface listing.  Default for Solaris and HPUX.",
+   "Traditional UNIX interface listing.  Default for Solaris, BSD and HPUX.",
    "${format}{check-existence}"
    "${ifdisplay?}{"
-   "${name}: flags=${flags}{number}{%hx}<${flags}{string}{,}>"
+   "${name}: flags=${flags}{number}{%x}<${flags}{string}{,}>"
+   "${metric?}{ metric ${metric}}"
    "${mtu?}{ mtu ${mtu}}${\\n}"
+   /* Print only if hwtype emits something.  */
+   "${exists?}{hwtype?}{"
+     "${hwtype?}{${\\t}${hwtype}${exists?}{hwaddr?}{"
+       "${hwaddr?}{ ${hwaddr}}}${\\n}}}"
+   "${exists?}{tunnel?}{"
+     "${tunnel?}{${\\t}tunnel inet ${tunsrc} --> ${tundst}${\\n}}}"
    "${addr?}{${\\t}inet ${addr}"
+   "${dstaddr?}{ --> ${dstaddr}}"
    " netmask ${netmask}{0}{%#02x}${netmask}{1}{%02x}"
    "${netmask}{2}{%02x}${netmask}{3}{%02x}"
    "${brdaddr?}{ broadcast ${brdaddr}}${\\n}}"
-   "${exists?}{hwtype?}{${hwtype?}{${\\t}${hwtype}"
-   "}${exists?}{hwaddr?}{${hwaddr?}{ ${hwaddr}}}${\\n}}"
+   "${media?}{${\\t}media: ${media}${\\n}}"
+   "${status?}{${\\t}status: ${status}${\\n}}"
    "}"
   },
   /* Resembles the output of ifconfig shipped with OSF 4.0g.  */
@@ -155,18 +186,32 @@ struct format formats[] = {
    "${format}{check-existence}"
    "${ifdisplay?}{"
    "${name}: flags=${flags}{number}{%x}<${flags}{string}{,}>${\\n}"
+   "${exists?}{tunnel?}{"
+     "${tunnel?}{${\\t}inet tunnel src ${tunsrc} tunnel dst ${tundst}${\\n}}}"
    "${addr?}{${\\t}inet ${addr}"
+   "${dstaddr?}{ --> ${dstaddr}}"
    " netmask ${netmask}{0}{%02x}${netmask}{1}{%02x}"
    "${netmask}{2}{%02x}${netmask}{3}{%02x}"
    "${brdaddr?}{ broadcast ${brdaddr}}" "${mtu?}{ ipmtu ${mtu}}${\\n}}"
+   "${exists?}{hwtype?}{"
+     "${hwtype?}{${\\t}${hwtype}${exists?}{hwaddr?}{"
+       "${hwaddr?}{ ${hwaddr}}}${\\n}}}"
    "}"
   },
+  {"check",
+   "Shorthand for `check-existence'.",
+   "${format}{check-existence}"
+  },
   /* If interface does not exist, print error message and exit. */
   {"check-existence",
    "If interface does not exist, print error message and exit.",
    "${index?}{}"
    "{${error}{${progname}: error: interface `${name}' does not exist${\\n}}"
    "${exit}{1}}"},
+  {"?",
+   "Synonym for `help'.",
+   "${format}{help}"
+  },
   {"help",
    "Display this help output.",
    "${foreachformat}{"
@@ -181,7 +226,7 @@ struct format formats[] = {
    "${newline}"
    "}"
    "${exit}{0}"},
-  {0, 0}
+  {NULL, NULL, NULL}
 };
 
 /* Default format.  */
@@ -199,37 +244,43 @@ enum {
 };
 
 static struct argp_option argp_options[] = {
+#define GRP 10
   { "verbose", 'v', NULL, 0,
-    "output information when configuring interface" },
+    "output information when configuring interface", GRP },
   { "all", 'a', NULL, 0,
-    "display all available interfaces" },
+    "display all available interfaces", GRP },
   { "interface", 'i', "NAME", 0,
-    "configure network interface NAME" },
+    "configure network interface NAME", GRP },
   { "address", 'A', "ADDR", 0,
-    "set interface address to ADDR" },
+    "set interface address to ADDR", GRP },
   { "netmask", 'm', "MASK", 0,
-    "set netmask to MASK" },
+    "set netmask to MASK", GRP },
   { "dstaddr", 'd', "ADDR", 0,
-    "set destination (peer) address to ADDR" },
-  { "peer", 'p', "ADDR", OPTION_ALIAS },
+    "set destination (peer) address to ADDR", GRP },
+  { "peer", 'p', "ADDR", OPTION_ALIAS,
+    "synonym for dstaddr", GRP },
   { "broadcast", 'B', "ADDR", 0,
-    "set broadcast address to ADDR" },
-  { "brdaddr", 'b', NULL, OPTION_ALIAS, }, /* FIXME: Do we really need it? */
+    "set broadcast address to ADDR", GRP },
+  { "brdaddr", 'b', NULL, OPTION_ALIAS,
+    "synonym for broadcast", GRP }, /* FIXME: Do we really need it? */
   { "mtu", 'M', "N", 0,
-    "set mtu of interface to N" },
+    "set mtu of interface to N", GRP },
   { "metric", METRIC_OPTION, "N", 0,
-    "set metric of interface to N" },
+    "set metric of interface to N", GRP },
   { "format", FORMAT_OPTION, "FORMAT", 0,
-    "select output format (or set back to default)" },
+    "select output format; set to `help' for info", GRP },
   { "up", UP_OPTION, NULL, 0,
-    "activate the interface (default if address is given)" },
+    "activate the interface", GRP },
   { "down", DOWN_OPTION, NULL, 0,
-    "shut the interface down" },
+    "shut the interface down", GRP },
   { "flags", 'F', "FLAG[,FLAG...]", 0,
-    "set interface flags" },
+    "set interface flags", GRP },
+  { "list", 'l', NULL, 0,
+    "list available or selected interfaces", GRP },
   { "short", 's', NULL, 0,
-    "short output format" },
-  { NULL }
+    "short output format", GRP },
+#undef GRP
+  { NULL, 0, NULL, 0, NULL, 0 }
 };
 
 const char doc[] = "Configure network interfaces.";
@@ -287,6 +338,7 @@ PARSE_OPT_SET_ADDR (address, address, ADDR)
 PARSE_OPT_SET_ADDR (netmask, netmask, NETMASK)
 PARSE_OPT_SET_ADDR (dstaddr, destination / peer address, DSTADDR)
 PARSE_OPT_SET_ADDR (brdaddr, broadcast address, BRDADDR)
+PARSE_OPT_SET_ADDR (hwaddr, hardware address, HWADDR)
 
 #define PARSE_OPT_SET_INT(field, fname, fvalid) \
 void								\
@@ -326,12 +378,24 @@ void parse_opt_set_af (struct ifconfig *ifp, char *af)
 }
 
 void
-parse_opt_set_flag (struct ifconfig *ifp, int flag, int rev)
+parse_opt_set_flag (struct ifconfig *ifp MAYBE_UNUSED,
+		    int flag, int rev)
 {
-  if (rev)
-    ifp->clrflags |= flag;
+  if (ifp)
+    ifp->valid |= IF_VALID_FLAGS;
   else
-    ifp->setflags |= flag;
+    pending_valid |= IF_VALID_FLAGS;
+
+  if (rev)
+    {
+      pending_clrflags |= flag;
+      pending_setflags &= ~flag;
+    }
+  else
+    {
+      pending_setflags |= flag;
+      pending_clrflags &= ~flag;
+    }
 }
 
 void
@@ -349,7 +413,8 @@ parse_opt_flag_list (struct ifconfig *ifp, const char *name)
 	len = strlen (name);
 
       if ((mask = if_nametoflag (name, len, &rev)) == 0)
-	error (EXIT_FAILURE, 0, "unknown flag %*.*s", len, len, name);
+	error (EXIT_FAILURE, 0, "unknown flag %*.*s",
+	       (int) len, (int) len, name);
       parse_opt_set_flag (ifp, mask, rev);
 
       name += len;
@@ -436,11 +501,28 @@ parse_opt_set_default_format_from_file (const char *file)
 void
 parse_opt_finalize (struct ifconfig *ifp)
 {
+  /* The flags `--up' and `--down' are allowed early.  */
+  if (ifp && pending_valid)
+    {
+      ifp->valid |= pending_valid;
+      pending_valid = 0;
+    }
+
+  /* Only the empty set of actions, i.e., only the interface name
+   * is present on the command line, merits printout of status.
+   */
   if (ifp && !ifp->valid)
     {
       ifp->valid = IF_VALID_FORMAT;
       ifp->format = default_format;
     }
+
+  if (ifp && (pending_setflags | pending_clrflags))
+    {
+      ifp->setflags |= pending_setflags;
+      ifp->clrflags |= pending_clrflags;
+      pending_setflags = pending_clrflags = 0;
+    }
 }
 
 static error_t
@@ -494,6 +576,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       parse_opt_set_default_format ("netstat");
       break;
 
+    case 'l':
+      list_mode++;
+      break;
+
     case 'v':
       verbose++;
       break;
@@ -525,7 +611,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
 }
 
 static char *
-default_help_filter (int key, const char *text, void *input)
+default_help_filter (int key, const char *text,
+		     void *input MAYBE_UNUSED)
 {
   char *s;
 
@@ -549,7 +636,8 @@ static struct argp argp =
     NULL,
     doc,
     NULL,
-    default_help_filter
+    default_help_filter,
+    NULL
   };
 
 static int
@@ -600,15 +688,15 @@ parse_cmdline (int argc, char *argv[])
 		   "can't get memory for interface configuration");
 	  ifp = &ifs[nifs - 1];
 	  *ifp = ifconfig_initializer;
-	  ifp->name = ifnxp->if_name;
+	  ifp->name = strdup (ifnxp->if_name);
+	  if (!ifs)
+	    error (EXIT_FAILURE, errno,
+		   "can't get memory for interface configuration name");
 	  ifp->valid = IF_VALID_FORMAT;
 	  ifp->format = default_format;
 	  ifnxp++;
 	}
-      /* XXX: We never do if_freenameindex (ifnx), because we are
-	 keeping the names for later instead using strdup
-	 (if->if_name) here.  */
-
+      if_freenameindex (ifnx);
       qsort (ifs, nifs, sizeof (ifs[0]), cmp_if_name);
     }
 }
