@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021
-  Free Software Foundation, Inc.
+  Copyright (C) 2001-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -40,13 +38,13 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <unused-parameter.h>
+#include <attribute.h>
 
 #include <ping.h>
 #include <ping_impl.h>
 
 int
-ping_router (char *hostname _GL_UNUSED_PARAMETER)
+ping_router (char *hostname MAYBE_UNUSED)
 {
   return 0;
 }
