@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2011, 2012 Free Software Foundation, Inc.
+  Copyright (C) 2011-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -66,8 +66,20 @@ typedef struct _code {
 #  define LOG_PRI(p)	((p) & LOG_PRIMASK)
 #endif
 
+/* Glibc prior to 2.17 included a definition of LOG_MAKEPRI
+ * that evaluated LOG_MAKEPRI(LOG_USER, 0) to (1 << 9).
+ * The first argument was shifted three bits, ignoring
+ * the definition LOG_USER = (1 << 3).  Avoid this
+ * harmful mistake.
+ */
+#ifdef LOG_MAKEPRI
+# if LOG_MAKEPRI (1, 0) > LOG_PRIMASK
+#  undef LOG_MAKEPRI
+# endif
+#endif /* LOG_MAKEPRI */
+
 #ifndef LOG_MAKEPRI
-#  define LOG_MAKEPRI(fac, p)	(((fac) << 3) | (p))
+#  define LOG_MAKEPRI(fac, p)	((fac) | (p))
 #endif
 
 #ifndef INTERNAL_NOPRI
@@ -75,7 +87,7 @@ typedef struct _code {
 #endif
 
 #ifndef INTERNAL_MARK	/* mark "facility", first unused value */
-#  define INTERNAL_MARK	LOG_MAKEPRI(LOG_NFACILITIES, 0)
+#  define INTERNAL_MARK	LOG_MAKEPRI(LOG_NFACILITIES << 3, 0)
 #endif
 
 #ifndef LOG_FACMASK
