@@ -1,8 +1,5 @@
 /*
-  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Free Software
-  Foundation, Inc.
+  Copyright (C) 1993-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -117,7 +114,7 @@
 #include <progname.h>
 #include <argp.h>
 #include <libinetutils.h>
-#include "unused-parameter.h"
+#include "attribute.h"
 #include "xalloc.h"
 
 /*
@@ -309,7 +306,7 @@ void prevent_routing (int fd, struct auth_data *ap);
 #endif
 
 void
-rlogind_sigchld (int signo _GL_UNUSED_PARAMETER)
+rlogind_sigchld (int signo MAYBE_UNUSED)
 {
   pid_t pid;
   int status;
@@ -488,7 +485,7 @@ static struct argp_option options[] = {
 
 static error_t
 parse_opt (int key, char *arg,
-	   struct argp_state *state _GL_UNUSED_PARAMETER)
+	   struct argp_state *state MAYBE_UNUSED)
 {
   switch (key)
     {
@@ -2230,7 +2227,7 @@ do_pam_check (int infd, struct auth_data *ap, const char *service)
 static int
 rlogin_conv (int num, const struct pam_message **pam_msg,
 	     struct pam_response **pam_resp,
-	     void *data _GL_UNUSED_PARAMETER)
+	     void *data MAYBE_UNUSED)
 {
   struct pam_response *resp;
 
