@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  Copyright (C) 1995-2022 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -731,13 +729,14 @@ printsub (char direction, unsigned char *pointer, int length)
 		}
 	      {
 		char tbuf[64];
-		sprintf (tbuf, "%s%s%s%s%s",
+
+		snprintf (tbuf, sizeof (tbuf), "%s%s%s%s%s",
 			 pointer[2] & MODE_EDIT ? "|EDIT" : "",
 			 pointer[2] & MODE_TRAPSIG ? "|TRAPSIG" : "",
 			 pointer[2] & MODE_SOFT_TAB ? "|SOFT_TAB" : "",
 			 pointer[2] & MODE_LIT_ECHO ? "|LIT_ECHO" : "",
 			 pointer[2] & MODE_ACK ? "|ACK" : "");
-		fprintf (NetTrace, "%s", tbuf[1] ? &tbuf[1] : "0");
+		fprintf (NetTrace, "%s", tbuf[0] ? &tbuf[1] : "0");
 	      }
 	      if (pointer[2] & ~(MODE_MASK))
 		fprintf (NetTrace, " (0x%x)", pointer[2]);
@@ -1061,7 +1060,6 @@ Exit (int returnCode)
 void
 ExitString (char *string, int returnCode)
 {
-  SetForExit ();
   fwrite (string, 1, strlen (string), stderr);
   exit (returnCode);
 }
